/*!
 * Copyright (c) Microsoft. All rights reserved.
 * Licensed under the MIT license. See LICENSE file in the project.
 */
import { Axes, GraphView, HighlightHoveredNode, LabelHoveredNode, HandleNodeClicks, Nodes, Edges, NodeSetHighlight, NodeSetLabel, Camera, } from '@graspologic/react';
import { SettingsPane, DisplaySettings, EdgeSettings, NodeSettings, } from '@graspologic/render-controls-react';
import * as React from 'react';
import { memo, useMemo, useState, useEffect, useCallback, } from 'react';
import useResizeObserver from 'use-resize-observer';
// import { LayoutSettings } from '@graspologic/layout-react'
import { DEFAULT_INSTANCE_ID, COMPONENT_ID } from '../interfaces';
import { GlobalStyles } from './GlobalStyles';
import { useColorizer } from './hooks/useColorizer';
import { useGraphData } from './hooks/useGraphData';
import { useHoverHandler } from './hooks/useHoverHandler';
import { usePositionChanges } from './hooks/usePositionChanges';
import { useVertexClickHandler } from './hooks/useVertexClickHandler';
import { useHighlightedVertices, useSelectedVertices } from './hooks/vac';
const styles = {
    brickWall: {
        height: '100%',
        width: '100%',
    },
};
export const Graph = memo(function GraphComponentInner({ manager, configuration, }) {
    const showControls = configuration.showControls != null ? configuration.showControls : true;
    const renderSettings = useMemo(() => configuration.rendering || {}, [
        configuration,
    ]);
    const controlsSettings = useMemo(() => configuration.controls || {}, [
        configuration,
    ]);
    const [api, setApi] = useState();
    const is3D = Boolean(renderSettings.is3D);
    const colorizer = useColorizer(manager, configuration);
    const graphData = useGraphData(manager, is3D);
    const handleVertexClick = useVertexClickHandler(manager);
    const highlighted = useHighlightedVertices(manager);
    const selected = useSelectedVertices(manager);
    const onHover = useHoverHandler(manager);
    const { ref } = useResizeObserver();
    usePositionChanges(api, configuration, manager);
    const instanceId = configuration.instanceId || DEFAULT_INSTANCE_ID;
    const renderConfig = api?.config;
    useEffect(() => {
        if (renderConfig) {
            // Load any existing config at startup
            const existingState = manager.getState(COMPONENT_ID, instanceId);
            if (existingState) {
                renderConfig.load(existingState);
            }
            // Set up a subscription
            const subscription = renderConfig.onChange.subscribe(() => {
                manager.setState(COMPONENT_ID, instanceId, renderConfig.copy());
            });
            return () => subscription.unsubscribe();
        }
    }, [renderConfig, instanceId, manager]);
    return (React.createElement(React.Fragment, null,
        React.createElement(GlobalStyles, null),
        React.createElement("div", { style: styles.brickWall, ref: ref },
            React.createElement(GraphView, { colorizer: colorizer, data: graphData, backgroundColor: [0, 0, 0, 0], is3D: is3D, ref: useCallback(ref => {
                    if (ref !== api) {
                        setApi(ref);
                    }
                }, [api]), style: styles.brickWall },
                React.createElement(Axes, { inCorner: renderSettings.cornerAxes, shown: renderSettings.drawAxes }),
                React.createElement(Camera, { interactive: true }),
                React.createElement(HandleNodeClicks, { onClick: handleVertexClick }),
                React.createElement(HighlightHoveredNode, { onHover: onHover }),
                React.createElement(LabelHoveredNode, null),
                React.createElement(NodeSetHighlight, { vertexIds: highlighted }),
                React.createElement(NodeSetLabel, { vertexIds: highlighted }),
                React.createElement(NodeSetHighlight, { vertexIds: selected }),
                React.createElement(NodeSetLabel, { vertexIds: selected }),
                React.createElement(Nodes, { shown: renderSettings.drawNodes, minRadius: renderSettings.nodeMinRadius, maxRadius: renderSettings.nodeMaxRadius, outline: renderSettings.nodeOutline, hideOnMove: renderSettings.hideNodesOnMove }),
                React.createElement(Edges, { shown: renderSettings.drawEdges, alpha: renderSettings.edgeAlpha, minWidth: renderSettings.edgeMinWidth, maxWidth: renderSettings.edgeMaxWidth, hideOnMove: renderSettings.hideNodesOnMove, depthWrite: renderSettings.edgeDepthWrite, antialias: renderSettings.edgeAntialias, constantWidth: renderSettings.constantEdgeWidth }),
                showControls ? (React.createElement(SettingsPane, null,
                    React.createElement(DisplaySettings, { interpolationTimeRange: controlsSettings.displayInterpolationTimeRange }),
                    React.createElement(NodeSettings, { minRadiusRange: controlsSettings.nodeMinRadiusRange, maxRadiusRange: controlsSettings.nodeMaxRadiusRange }),
                    React.createElement(EdgeSettings, { alphaRange: controlsSettings.edgeAlphaRange, minWidthRange: controlsSettings.edgeMinWidthRange, maxWidthRange: controlsSettings.edgeMaxWidthRange }))) : null))));
});
